@@ -1,3 +1,4 @@
+use std::cmp::Ordering;
 use std::fs::File;
 use std::io::prelude::*;
 use std::io::Write;
@@ -12,6 +13,7 @@ pub struct Buffer {
     offset: usize,
     pos: Vec2,
     size: Vec2,
+    line_num: bool,
 }
 
 #[derive(PartialEq)]
@@ -22,13 +24,14 @@ enum MoveForwardOutcome {
 }
 
 impl Buffer {
-    pub fn from_lines(lines: Vec<String>, pos: Vec2, size: Vec2) -> Self {
+    pub fn from_lines(lines: Vec<String>, pos: Vec2, size: Vec2, line_num: bool) -> Self {
         Self {
             lines,
             cursor: Vec2::default(),
             offset: 0,
             size,
             pos,
+            line_num,
         }
     }
 
@@ -44,10 +47,12 @@ impl Buffer {
             vec![]
         };
 
-        Self::from_lines(lines, pos, size)
+        Self::from_lines(lines, pos, size, true)
     }
 
     pub fn render(&self, term: &mut impl Write) {
+        let line_num_size = self.lines.len().to_string().len();
+
         let mut row = 0;
         let mut col = 0;
         let mut cursor: Option<Vec2> = None;
@@ -59,14 +64,29 @@ impl Buffer {
             .take(self.size.y)
             .enumerate()
         {
+            let line_num = y + self.offset + 1;
+
+            let relative_line_num = match self.cursor.y.cmp(&(y + self.offset)) {
+                Ordering::Greater => self.cursor.y - (y + self.offset),
+                Ordering::Less => (y + self.offset) - self.cursor.y,
+                Ordering::Equal => line_num,
+            };
+
+            let line_num_str = format!(
+                "{:>width$}",
+                relative_line_num.to_string(),
+                width = line_num_size
+            );
+
             if line.is_empty() {
                 write!(
                     term,
-                    "{}{}",
+                    "{}{}{} ",
                     termion::cursor::Goto(
                         (self.pos.x + col + 1) as u16,
                         (self.pos.y + row + 1) as u16
                     ),
+                    line_num_str,
                     termion::clear::UntilNewline
                 )
                 .unwrap();
@@ -86,11 +106,25 @@ impl Buffer {
                     cursor = Some(Vec2::new(col, row));
                 }
 
+                if x == 0 && self.line_num {
+                    write!(
+                        term,
+                        "{}{} ",
+                        termion::cursor::Goto(
+                            (self.pos.x + col + 1) as u16,
+                            (self.pos.y + row + 1) as u16
+                        ),
+                        line_num_str
+                    )
+                    .unwrap();
+                }
+
                 write!(
                     term,
                     "{}{}",
                     termion::cursor::Goto(
-                        (self.pos.x + col + 1) as u16,
+                        ((if self.line_num { line_num_size + 1 } else { 0 }) + self.pos.x + col + 1)
+                            as u16,
                         (self.pos.y + row + 1) as u16
                     ),
                     c
@@ -117,7 +151,10 @@ impl Buffer {
                 term,
                 "{}",
                 termion::cursor::Goto(
-                    (self.pos.x + cursor.x + 1) as u16,
+                    ((if self.line_num { line_num_size + 1 } else { 0 })
+                        + self.pos.x
+                        + cursor.x
+                        + 1) as u16,
                     (self.pos.y + cursor.y + 1) as u16
                 )
             )
@@ -357,6 +394,7 @@ mod tests {
             offset: 0,
             pos: Vec2::default(),
             size: Vec2::new(100, 100),
+            line_num: true,
         };
 
         buffer.move_cursor_down();
@@ -372,6 +410,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.move_cursor_down();
@@ -388,6 +427,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.move_cursor_down();
@@ -403,6 +443,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.move_cursor_up();
@@ -418,6 +459,7 @@ mod tests {
             offset: 0,
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
+            line_num: true,
         };
 
         buffer.move_forward();
@@ -433,6 +475,7 @@ mod tests {
             offset: 0,
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
+            line_num: true,
         };
 
         buffer.move_forward();
@@ -448,6 +491,7 @@ mod tests {
             offset: 0,
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
+            line_num: true,
         };
 
         buffer.move_forward();
@@ -463,6 +507,7 @@ mod tests {
             offset: 0,
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
+            line_num: true,
         };
 
         buffer.move_forward();
@@ -479,6 +524,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -494,6 +540,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -508,6 +555,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -522,6 +570,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -536,6 +585,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -550,6 +600,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -562,6 +613,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -574,6 +626,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -586,6 +639,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -598,6 +652,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -613,6 +668,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -627,6 +683,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -641,6 +698,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -659,6 +717,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.word_forward();
@@ -674,6 +733,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_char('a');
@@ -690,6 +750,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_char('a');
@@ -706,6 +767,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_line_after_cursor("hello".to_string());
@@ -726,6 +788,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_line_after_cursor("inserted".to_string());
@@ -746,6 +809,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_new_line();
@@ -762,6 +826,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_new_line();
@@ -782,6 +847,7 @@ mod tests {
             size: Vec2::new(100, 1),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.insert_new_line();
@@ -798,6 +864,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.join_line();
@@ -814,6 +881,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.backspace();
@@ -830,6 +898,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.backspace();
@@ -846,6 +915,7 @@ mod tests {
             size: Vec2::new(100, 100),
             pos: Vec2::default(),
             offset: 0,
+            line_num: true,
         };
 
         buffer.backspace();
